require 'git'

module Speedflow
  module Plugin
    module Git
      # Git client
      class Client
        # @return [Prompt] Prompt.
        attr_writer :prompt

        # @return [Git::Client] Git client.
        attr_writer :git_client

        # Initialize.
        #
        # config - Speedflow::Plugin::Git::Configuration instance.
        # prompt - Speedflow::Plugin::Git::Prompt instance.
        #
        # Examples
        #
        #    Client.new({}, Speedflow::Plugin::Git::Prompt.new)
        #    # => <Speedflow::Plugin::Git::Client>
        #
        # Returns nothing.
        def initialize(config, prompt)
          @config = config
          @prompt = prompt
        end

        # Public: Create branch.
        #
        # branch      - Branch.
        # branch_from - Branch from.
        #
        # Returns Git::Branch.
        def create_branch(branch, branch_from)
          safe do
            git = git_client.open('.')
            git.branch(branch_from).checkout unless branch_from.empty?
            git.branch(branch).checkout
          end
        end

        # Public: Branches.
        #
        # branch_type - String of type (:local or :remote)
        #
        # Returns Array of local branches.
        def branches(branch_type = :local)
          safe do
            git = git_client.open('.')
            git.branches.send(branch_type)
          end
        end

        # Public: Safe process Git action.
        #
        # Returns nothing.
        def safe
          yield
        rescue StandardError => exception
          prompt.errors exception
          abort
        end

        # Public: Git client.
        #
        # Returns ::Git instance.
        def git_client
          @git_client ||= ::Git
        end

        # Public: Prompt.
        #
        # Returns ::Speedflow::Plugin::Git::Prompt instance.
        def prompt
          @prompt ||= ::Speedflow::Plugin::Git::Prompt.new
        end
      end
    end
  end
end
