module Speedflow
  module Plugin
    module Git
      # Plugin core
      class Plugin < Speedflow::Plugin::Abstract
        # @return [Prompt] Plugin prompt.
        attr_writer :prompt

        # @return [Client] Plugin client.
        attr_writer :client

        # Public: Constructor
        #
        # config - Speedflow::Plugin::Configuration object.
        # prompt - Speedflow::Plugin::Prompt object.
        #
        # Examples
        #
        #    Manager.new(
        #      <Speedflow::Plugin::Configuration.new({})>,
        #      <Speedflow::Plugin::Prompt.new>)
        #    # => <Speedflow::Plugin::Abstract>
        #
        # Returns nothing.
        def initialize(config, _)
          super(config, Prompt.new)
        end

        # Public: Create branch.
        #
        # Returns Hash of branch.
        def action_create_branch
          branch = config.by_input('branch')
          branch = prompt.ask_branch if branch.empty?

          branch_from = config.by_input('branch_from')
          branch_from = action_search_branch['branch'] if branch_from.empty?

          client.create_branch(branch, branch_from)

          prompt.ok '[GIT] Branch successful created: ' \
                    "'#{branch}' from '#{branch_from}'."

          { 'branch' => branch, 'branch_from' => branch_from }
        end

        # Public: Search branch.
        #
        # Returns Hash of branch.
        def action_search_branch
          branch = prompt.branch(client.branches(prompt.branch_type))
          action_search_branch if branch == :retry

          { 'branch' => branch }
        end

        # Public: Plugin client.
        #
        # Returns Client instance.
        def client
          @client ||= Client.new(@config, @prompt)
        end

        # Public: Plugin prompt.
        #
        # Returns Prompt instance.
        def prompt
          @prompt ||= Prompt.new
        end
      end
    end
  end
end
