module Speedflow
  module Plugin
    module Git
      # Plugin prompt
      class Prompt
        # @return [<Speedflow::Plugin::Prompt>] Plugin prompt.
        attr_writer :prompt

        # Public: Prompt ask branch.
        #
        # Returns String of title.
        def ask_branch
          ask('What is the name of branch?', required: true)
        end

        # Public: Prompt branch.
        #
        # branches - List of branches.
        #
        # Returns branch name.
        def branch(branches)
          prompt.select('Choose the branch:', required: true) do |menu|
            menu.choice 'Re-select branch type', :retry
            branches.each do |branch|
              next if branch.name.include? '->'
              branch_full = branch.name
              branch_full = "#{branch.remote}/#{branch.name}" if branch.remote
              menu.choice branch.name, branch_full
            end
          end
        end

        # Public: Prompt branch type.
        #
        # Returns String of type (:local or :remote).
        def branch_type
          prompt.select('Choose the branch type:', required: true) do |menu|
            menu.choice 'Local branches', :local
            menu.choice 'Remote branches', :remote
          end
        end

        # Public: Errors from Git exception.
        #
        # exception - Git exception.
        #
        # Returns nothing.
        def errors(exception)
          prompt.error 'Git errors'
          prompt.warn exception.message
        end

        # Delegate.
        #
        # method - Method.
        # args   - Arguments.
        # block  - Block.
        #
        # Returns wathever.
        def method_missing(method, *args, &block)
          prompt.send(method, *args, &block)
        end

        # Public: TTY prompt.
        #
        # Returns ::Speedflow::Plugin::Prompt instance.
        def prompt
          @prompt ||= ::Speedflow::Plugin::Prompt.new
        end
      end
    end
  end
end
